<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Http\V1\Auth;

use App\Repositories\Enums\ResponseCodeEnum;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use App\Http\V1\Controller;
use App\Services\AuthService;
use App\Services\RoleAndPermissionsService;

class AuthorizationController extends Controller
{

    protected $authService = null;

    /**
     * Create a new AuthController instance.
     * @param \App\Services\AuthService $authService
     */
    public function __construct(AuthService $authService)
    {
        $this->authService = $authService;
    }

    /**
     * @param \Illuminate\Http\Request $request
     */
    public function login(Request $request)
    {
        $message = [
            'username.required' => "请输入用户名",
            'password.required' => "请输入密码",
        ];
        $this->validateRequest($request, $message);

        try{
            $response = $this->authService->login($request);
            return Response::success([
                'token' => $response,
            ], '登陆成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    /**
     * @param \Illuminate\Http\Request $request
     */
    public function logout(Request $request)
    {
        try{
            $this->authService->logout($request);
            return Response::success(null,'请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }


    public function addUser(Request $request)
    {
        $message = [
            'username.required' => "用户登陆账号必填",
            'password.required' => "密码必填",
            'type.required' => "用户类型必选",
            'nickname.required' => "用户名称必填",
            'email.required' => "邮箱必填",
        ];
        $this->validateRequest($request, $message);
        try{
            $this->authService->saveOrUpdateUser($request);
            return Response::success(null,'请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    public function editUser(Request $request)
    {
        $message = [
            'type.required' => "用户类型必选",
            'nickname.required' => "用户名称必填",
            'email.required' => "邮箱必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $this->authService->saveOrUpdateUser($request);
            return Response::success(null,'请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }


    public function changeUserStatus(Request $request, $id)
    {
       if(!$id){
           return Response::fail('缺少id参数',500);
       }

       try{
            $this->authService->updateUserStatus($request,$id);
            return Response::success(null,'请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }

    }

    public function delUserByIds(Request $request)
    {
        $message = [
            'ids.required' => "用户id必传",
        ];
        $this->validateRequest($request, $message);

        try{
            $this->authService->deleteUser($request);
            return Response::success(null,'请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    /**
     * 获取用户信息
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function info(Request $request)
    {
        try{
            $user = $this->authService->getUserInfo();
            $service = app(RoleAndPermissionsService::class);
            $service->getRolesToUser();
            return Response::success([
                'userinfo' => $user,
            ], '请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    public function listByPage(Request $request)
    {
        try{
            $service = app(AuthService::class);
            $roles = $service->listByPage($request);
            return Response::success($service->formatKeysfromArray($roles,'toCamelCase'), '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }


    /**
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function resetPassword(Request $request)
    {
        $message = [
            'oldpassword.required' => "请输入旧密码",
            'newpassword.required' => "请输入新密码",
        ];
        $this->validateRequest($request, $message);
        try{
            $response = $this->authService->resetPassword($request);
            return Response::success([], '更新成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    /**
     * 获取菜单列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|void
     */
    public function getMenuList(Request $request)
    {
        try{
            $service = app(RoleAndPermissionsService::class);
            $menuList = $service->getUserRoleMenus();
            return Response::success([
                'data' => $service->getMenusTree($menuList,'page'),
            ], '请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    /**
     * 获取所有菜单列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function getAllMenuList(Request $request)
    {
        try{
            $service = app(RoleAndPermissionsService::class);
            $menuList = $service->getAllMenus();
            $treeMenu = $service->getMenusTree($menuList);
            return Response::success([
                'data' => $service->formatKeysfromArray($treeMenu,'toCamelCase'),
            ], '请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    /**
     * 更新菜单
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function editMenus(Request $request)
    {
        $message = [
            'id.required' => "菜单id必选",
            'name.required' => "菜单名称必填",
            'path.required' => "菜单路径必填",
            'icon.required' => "菜单icon必填",
            'parentId.required' => "上级菜单必选",
            'menuType.required' => "菜单类型必选",
            'component.required' => "菜单前端组件必填",
            'status.required' => "菜单状态必选",
            'sort.required' => "菜单排序必填",
        ];
        $this->validateRequest($request, $message);

        if($request->get('selected') != true){
            return Response::fail('该菜单未被选中编辑');
        }

        try{
            $service = app(RoleAndPermissionsService::class);
            $service->editMenu($request);
            return Response::success([], '更新成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    public function getDefaultButtonPermission(Request $request)
    {
        try{
            $service = app(RoleAndPermissionsService::class);
            $buttonPermissions = $service->getButtonDefaultPermission();
            return Response::success($buttonPermissions, '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    public function addSubMenus(Request $request)
    {
        $message = [
            'type.required' => "菜单类型必选",
            'parentId.required' => "上级菜单必填",
            'level.required' => "菜单深度必填",
            'sort.required' => "菜单排序必填",
            'status.required' => "菜单状态必选",
            'path.required' => "菜单路径必填",
            'title.required' => "菜单名称必填",
        ];
        $this->validateRequest($request, $message);

        if('page' == $request->get('type') && $request->get('level') == 2){
            if(empty($request->get('icon'))){
                return Response::fail('菜单图标必选');
            }
            if(empty($request->get('component'))){
                return Response::fail('菜单前端组件必填');
            }
        }

        if('button' == $request->get('type') && $request->get('level') == 3){
            if(empty($request->get('buttonType'))){
                return Response::fail('菜单行为必选');
            }
        }

        try{
            $service = app(RoleAndPermissionsService::class);
            $service->addSubMenu($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    public function deleteMenus(Request $request)
    {
        $message = [
            'ids.required' => "菜单id必选",
        ];
        $this->validateRequest($request, $message);

        try{
            $service = app(RoleAndPermissionsService::class);
            $service->deleteMenus($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    /**
     * 获取角色列表（页面列表显示）
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function getAllByPage(Request $request)
    {
        try{
            $service = app(RoleAndPermissionsService::class);
            $roles = $service->getAllRolesToPage($request);
            return Response::success($service->formatKeysfromArray($roles,'toCamelCase'), '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    public function getAllList(Request $request)
    {
        try{
            $service = app(RoleAndPermissionsService::class);
            $roles = $service->getAllRolesList($request);
            return Response::success($service->formatKeysfromArray($roles,'toCamelCase'), '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    public function addRole(Request $request)
    {
        $message = [
            'name.required' => "角色名称必填",
            'remark.required' => "角色描述必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $service = app(RoleAndPermissionsService::class);
            $service->saveRole($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    /**
     * 编辑权限
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function editRole(Request $request)
    {
        $message = [
            'id.required' => "角色id必填",
            'name.required' => "角色名称必填",
            'remark.required' => "角色描述必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $service = app(RoleAndPermissionsService::class);
            $service->saveRole($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    /**
     * 分配角色权限
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function editRolePermission(Request $request)
    {
        $message = [
            'roleId.required' => "角色id必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $service = app(RoleAndPermissionsService::class);
            $service->syncPermissionToRole($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    public function delByIds(Request $request)
    {
        $message = [
            'ids.required' => "角色id必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $service = app(RoleAndPermissionsService::class);
            $service->deleteRoleByIds($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

}
