<?php /*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services;

use App\Repositories\Models\Menus;
use App\Support\Traits\Helpers;
use Illuminate\Http\Request;
use Spatie\Permission\Exceptions\RoleDoesNotExist;
use Spatie\Permission\Models\Permission;
use Spatie\Permission\Models\Role;
use App\Repositories\Contracts\MenusRepository;
use App\Repositories\Contracts\RolesRepository;


class RoleAndPermissionsService
{
    use Helpers;

    protected $user = null;
    protected $menusRepository = null;

    public function __construct(MenusRepository $menusRepository)
    {
        $this->user = auth()->user();
        $this->menusRepository = $menusRepository;
    }

    /**
     * 获取管理后台用户的所在的用户组
     */
    public function getRolesToUser($user = null){
        if($user == null){
            $user = $this->user;
        }
        $this->getAllPermissions($user);
        $role = $user->roles;
        return $role;
    }

    public function getUserPermissions(){
        $role = $this->getAllPermissions($this->user);
    }

    /**
     * 获取用户所有权限
     * @param $user
     * @param bool $parseUserRole
     * @return mixed
     */
    protected function getAllPermissions($user, $parseUserRole = true)
    {
        $role = $user->getpermissionsviaroles();
        if($parseUserRole){
            $newRoles = collect($user['roles'])->map(function($role) {
                $roleArray = $role->toArray();
                unset($roleArray['pivot']);
                unset($roleArray['guard_name']);
                $newPermission = collect($roleArray['permissions'])->map(function ($permission)  {
                    // 隐藏系统默认权限
                    if ($permission['menu_id'] > 0) {
                        unset($permission['pivot']);
                        unset($permission['guard_name']);
                        //$newPermission[] = $permission;
                        return $permission;
                    }
                });
                $roleArray['permissions'] = array_values(array_filter($newPermission->toArray()));

                return $roleArray;
            });
            unset($user['roles']);
            $user['roles'] = $newRoles->toArray();
            //将权限赋予当前用户
            auth()->setUser($user);
        }

        return $role;
    }


    /**
     * 获取系统默认级别的按钮权限类型
     * @return array|\Illuminate\Database\Eloquent\Builder[]|\Illuminate\Database\Eloquent\Collection
     */
    public function getButtonDefaultPermission()
    {
        $allPermission = Permission::query()->where(['menu_id' => 0, 'sys_default' => 1,'guard_name' => config('auth.defaults.guard')])->get();
        if($allPermission){
            collect($allPermission)->map(function($permission){
                $permission->title = $permission->name;
            });
            $allPermission = $this->formatKeysfromArray($allPermission);
        }

        return $allPermission;
    }

    /**
     * 获取用户授权菜单列表
     * @return array
     */
    public function getUserRoleMenus()
    {
        $permissions = $this->getAllPermissions($this->user);
        $menuIds = collect($permissions)->map(function($permission){
            if($permission instanceof \Spatie\Permission\Models\Permission){
                $permission = $permission->toArray();
            }
            return $permission['menu_id'];
        });

        $menuModel = app(Menus::class);
        $menuList = $menuModel
            ->whereIn($menuModel->getKeyName(),$menuIds)
            ->where('status',1)->get()->toArray();

        $menuList = collect($menuList)->map(function($menu){
            $menu['showAlways'] = true;
            return $menu;
        });

        return empty($menuList) || NULL ? [] : $menuList;
    }

    /**
     * 获取所有菜单列表
     * @return array
     * @throws \Psr\Container\ContainerExceptionInterface
     *
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function getAllMenus()
    {
        $menuModel = app(Menus::class);
        $allMenus = $menuModel->get();
        // key_name转换
        $parent_id = [];
        $menusList = collect($allMenus)->map(function($menu) use (&$parent_id){

            $newMenu = [
                'name' => $menu->menu_name,
                'title' => $menu->menu_name,
                'path' => $menu->menu_path,
                'icon' => $menu->menu_icon,
                'expand' => true,
                'checked' => false
            ];

            if($menu->parent_id > 0 && $menu->menu_type != 'top'){
                $parent_id[] = $menu->parent_id;
            }

            $menuArr = $menu->toArray();

            unset($menuArr['menu_name']);
            unset($menuArr['menu_path']);
            unset($menuArr['menu_icon']);

            return array_merge($newMenu,$menuArr);
        });

        $parentMenu = $menuModel->whereIn('id',$parent_id)->get();
        $menusList = collect($menusList)->map(function($menu) use ($parentMenu){
            $parent_id = $menu['parent_id'];
            if($parent_id > 0){
                $parentName = collect($parentMenu)->map(function($m) use (&$menu,$parent_id){
                   if($m->id == $parent_id){
                       $menu['parent_title'] = $m->menu_name;
                   }
                });
            }
            return $menu;
        });

        return $menusList->toArray();
    }

    /**
     * 获取菜单结构树
     * level : all (全部) top (只显示顶部) page (只显示2级,不包括button级)
     * @param $menus
     * @return false|mixed
     */
    public function getMenusTree($menus, $level = 'all')
    {
        $treeMenus = [];
        collect($menus)->map(function($menu) use (&$treeMenus, $level){
            if($menu['parent_id'] == -1 && 'top' == $menu['menu_type']){
                $treeMenus[$menu['id']] = $menu;
                $treeMenus[$menu['id']]['level'] = 0;
            }else{
                if('top' == $level){
                    return ;
                }elseif('page' == $level){
                   if('page' == $menu['menu_type']) {
                       if($menu['parent_id'] == 0) {
                           $treeMenus[$menu['id']] = $menu;
                           $treeMenus[$menu['id']]['level'] = 1;
                       }else{
                           $treeMenus = $this->getChildrenTree($treeMenus,$menu);
                       }
                   }
                }else{
                    if($menu['parent_id'] == 0) {
                        $treeMenus[$menu['id']] = $menu;
                        $treeMenus[$menu['id']]['level'] = 1;
                    }else{
                        $treeMenus = $this->getChildrenTree($treeMenus,$menu);
                    }
                }
            }
        });

        return array_values($treeMenus);
    }

    /**
     * 递归树形结构菜单
     * @param $treeMenus
     * @param $currMenu
     * @return array
     */
    private function getChildrenTree($treeMenus,$currMenu) : array
    {
        $returnTreeMenus = collect($treeMenus)->map(function($tMenus) use ($currMenu){
            if(!isset($tMenus['children']) || empty($tMenus['children'])){
                if($currMenu['parent_id'] == $tMenus['id']){
                    $currMenu['level'] = (int)$tMenus['level'] + 1;
                    $tMenus['children'][] = $currMenu;
                }
            } // 如果有children的话，进行递归循环
            else{
                if($currMenu['parent_id'] == $tMenus['id']){
                    $currMenu['level'] = (int)$tMenus['level'] + 1;
                    $tMenus['children'][] = $currMenu;
                }else{
                    $tMenus['children'] = $this->getChildrenTree($tMenus['children'],$currMenu);
                }
            }

            return $tMenus;
        });

        return $returnTreeMenus->toArray();
    }


    /**
     * 编辑菜单
     * @param Request $request
     * @return mixed
     */
    public function editMenu(Request $request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $menuModel = app(Menus::class);
        $menu = $menuModel->where('id',$params['id'])->first();
        if(!$menu){
            throw new \LogicException('此菜单不存在！',500);
        }
        $parentMenu = $menuModel->where('id',$params['parent_id'])->first();
        if(!$parentMenu){
            throw new \LogicException('上级菜单不存在！',500);
        }
        if(!in_array($params['menu_type'],['top','page','button'])){
            throw new \LogicException('菜单类型不正确！',500);
        }

        $params['menu_name'] = $params['title'];
        $params['menu_path'] = $params['path'];
        $params['menu_icon'] = $params['icon'];

        // 根据role规则重组 user
        $updateMenus = [];
        collect($this->menusRepository->rules['update'])->map(function($rule,$key) use ($params, &$updateMenus){
            if($key != 'id'){
                $updateMenus[$key] = $params[$key];
            }
        });
        $updateMenus['created_by'] = $this->user->id;
        $result = $this->menusRepository->update($updateMenus,$params['id']);

        return $result;
    }

    /**
     * 添加子菜单
     * @param $request
     * @return bool
     */
    public function addSubMenu($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $menuModel = app(Menus::class);

        $parentMenu = $menuModel->where('id',$params['parent_id'])->first();
        if(!$parentMenu){
            throw new \LogicException('上级菜单不存在！',500);
        }

        if('button' == $params['type']){
            $permission = Permission::query()->where(['menu_id' => 0, 'sys_default' => 1,'guard_name' => config('auth.defaults.guard'), 'action' => $params['button_type']])->first();
            if(!$permission){
                throw new \LogicException('按钮动作不存在！',500);
            }
        }

        $filterParams = [];
        collect($this->menusRepository->rules['create'])->map(function($rule,$key) use ($params, &$filterParams){
            if(isset($params[$key]) && !empty($params[$key])){
                $filterParams[$key] = $params[$key];
            } else{
                $filterParams[$key] = '';
            }
        });
        $filterParams['created_by'] = $this->user->id;
        $filterParams['menu_icon'] = $params['icon'];
        $filterParams['component'] = $params['component'];
        $filterParams['status'] = intval($params['status']);

        $filterParams = collect($filterParams)->map(function($value,$key) use ($params){
            if($value == ''){
                switch($key){
                    case 'menu_name' : $value = $params['title']; break;
                    case 'menu_path' : $value = $params['path']; break;
                    case 'menu_icon' : $value = $params['icon']; break;
                    case 'menu_type' : $value = $params['type']; break;
                    case 'is_show' : $value = 1; break;
                    case 'sys_default' : $value = 0; break;
                }
            }
            return $value;
        });

        $result = $this->menusRepository->create($filterParams->toArray());
        if($result){
            $permission = [];
            $permission['name'] = $result['menu_name'];
            $permission['menu_id'] = $result['id'];
            $permission['action'] = $params['button_type'];
            $permission['permission_type'] = 'menu';
            $permission['sys_default'] = 0;
            $permission['remark'] = $result['menu_name'].'的相关权限';
            $permission['guard_name'] = config('auth.defaults.guard');
            //增加对应的按钮权限
            $permissionResult = Permission::create($permission);
            // 如果是管理员，自动绑定赋予权限
            if($this->user->isSuperAdmin()){
                if($permissionResult){
                   $this->assignRoleAndPermissionToUser($permissionResult,$this->user);
                }
            }
        }
        return true;
    }

    public function deleteMenus($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        $menuModel = app(Menus::class);
        $menu_ids = explode(',',$params['ids']);
        if(!$menus = $menuModel->whereIn('id',$menu_ids)->get()){
            throw new \LogicException('选中的菜单不存在!',500);
        }
        //判断该菜单下面是否有其他子菜单
        /*if($menuModel->whereIn('parent_id',$menu_ids)->get()){
            throw new \LogicException('选中的菜单中有子菜单存在，无法删除该菜单。',500);
        }*/
        //判断选中的菜单是否已被赋予权限给到用户组
        //查询该菜单对应的权限数据
        $menuPermission = Permission::query()->whereIn('menu_id',$menu_ids)->get();
        if($menuPermission){
            $allRoles = Role::all();
            if($allRoles){
                $hasPermission = false;
                collect($allRoles)->map(function($role) use ($menuPermission, &$hasPermission){
                    collect($menuPermission)->map(function($permission) use ($role, &$hasPermission){
                        if($role->hasPermissionTo($permission)){
                            $hasPermission = true ;
                        }
                    });
                });
                if($hasPermission){
                    throw new \LogicException('用户权限中已经包含该菜单权限，请先取消角色中该菜单的权限，在进行删除操作.',500);
                }
                //删除对应的菜单
                collect($menus)->map(function($menu) use ($menuPermission){
                    $menu->delete();
                    //删除对应的菜单权限
                    collect($menuPermission)->map(function($permission){
                        $permission->delete();
                    });
                });
            }
        }

    }

    /**
     * 同步指定权限给指定用户
     * @param $permission
     * @param $user
     */
    public function assignRoleAndPermissionToUser($permission, $user)
    {
        $roles = $this->getRolesToUser($user);
        if($roles == null){
            throw new \LogicException('该用户没有所属用户组,请手动进行分组在分配权限!');
        }
        $roleArr = collect($roles)->map(function($role){
            return $role['id'];
        });
        // 获取当前用户的组信息
        $allRoles = Role::query()->whereIn( 'id' , $roleArr->toArray())->get();
        if($allRoles){
            collect($allRoles)->map(function($role) use ($permission,$user){
                $role->givePermissionTo($permission);
                $user->givePermissionTo($permission);
            });
        }
    }

    /**
     * 获取所有角色并获取对应的所有权限(用于列表展示)
     * @param $request
     * @return array
     */
    public function getAllRolesToPage($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        if(!$params['page_size'] || $params['page_size'] == 0){
           $params['page_size'] = 10;
        }
        $roles = app(RolesRepository::class)->paginate($params['page_size'],['id','name','remark','created_at','updated_at']);
        if($roles){
            $rolePermission = [];
            $items = collect($roles->items())->map(function($role) use (&$rolePermission){
                $permissions = $role->permissions->toArray();
                $permissions = collect($permissions)->map(function($permission){
                    if($permission['sys_default'] == 1 && $permission['menu_id'] == 0){
                        unset($permission);
                    }else{
                        unset($permission['pivot']);
                        $permission['created_at'] = date('Y-m-d H:i:s',strtotime($permission['created_at']));
                        $permission['updated_at'] = date('Y-m-d H:i:s',strtotime($permission['updated_at']));
                        return $permission;
                    }
                });
                $rolePermission[$role->id] = array_values(array_filter($permissions->toArray()));

                return $role;
            });

            $allItems = collect($items)->map(function($item) use ($rolePermission){
                $itemArr = $item->toArray();
                unset($itemArr['permissions']);
                $itemArr['permissions'] = $rolePermission[$item['id']];

                return $itemArr;
            });

            // 获取当前角色的所有权限
            $returnRecord['data'] = $this->formatKeysfromArray($allItems);
            $returnRecord['current_page'] = $roles->currentPage();
            $returnRecord['total'] = $roles->total();
        }
        return $returnRecord;
    }

    /**
     * 仅获取所有角色列表
     * @param $request
     * @return array
     */
    public function getAllRolesList($request)
    {
        $roles = Role::all();
        return $roles->toArray();
    }

    /**
     * 保存角色
     * @param $request
     * @return bool
     */
    public function saveRole($request){
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        try{
            $role = Role::query()->where('name', $params['name'])->first();
            if(!empty($role)) {
                $role = $role->toArray();
            }
            if(!isset($params['id']) || empty($params['id'])){
                if(!empty($role)){
                    throw new \LogicException('角色名称不能重复',500);
                }
            }else{
                if(!empty($role) && $role['id'] != $params['id']){
                    throw new \LogicException('角色名称不能重复',500);
                }
            }
        }catch(\Exception $exception){
            if (!$exception instanceof RoleDoesNotExist){
                throw $exception;
            }
        }
        app(RolesRepository::class)->saveRole($params);

        return true;
    }

    /**
     * 分配权限给相应角色
     * @param $request
     * @return bool
     */
    public function syncPermissionToRole($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        //判断该角色是否存在
        $role = Role::findById($params['role_id']);
        if(!$role){
            throw new \LogicException('角色不存在');
        }
        if(isset($params['perm_ids']) && !empty($params['perm_ids'])){
            $permisson_ids = explode(',',$params['perm_ids']);
            $permission = Permission::query()->whereIn('menu_id',$permisson_ids)
                ->where(['guard_name' => config('auth.defaults.guard'),'sys_default' => 0])->get();
            if(count($permission) != count($permisson_ids)){
                throw new \LogicException('部分权限不存在');
            }
            if(count($role->permissions->toArray()) > 0){
                //先作废原先的权限
                $role->revokePermissionTo($role->permissions);
            }
            $role->givePermissionTo($permission);
        }else{
            $role->revokePermissionTo($role->permissions);
        }

        return true;
    }

    /**
     * 删除角色
     * @param $request
     */
    public function deleteRoleByIds($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        //判断该角色是否存在
        $roles = Role::query()->whereIn('id',explode(',',$params['ids']))->get();
        if(!$roles){
            throw new \LogicException('角色不存在');
        }
        collect($roles)->map(function($role){
            $role->delete();
            if(count($role->permissions->toArray()) > 0){
                $role->revokePermissionTo($role->permissions);
            }
        });
    }
}
